part of katana_routing;

/// Color definition for the theme.
class ThemeColor {
  /// Color definition for the theme.
  const ThemeColor({
    this.primary = Colors.blue,
    this.secondary = Colors.cyan,
    this.primaryContainer = Colors.blueAccent,
    this.secondaryContainer = Colors.cyanAccent,
    this.disabled = Colors.grey,
    this.weak = Colors.grey,
    this.error = Colors.red,
    this.warning = Colors.amber,
    this.info = Colors.blue,
    this.success = Colors.green,
    this.surface = const Color(0xFFFFFFFF),
    this.background = const Color(0xFFFFFFFF),
    this.onPrimary = const Color(0xFFFFFFFF),
    this.onSecondary = const Color(0xFFFFFFFF),
    this.onDisabled = const Color(0xFFFFFFFF),
    this.onSurface = const Color(0xFFFFFFFF),
    this.onBackground = const Color(0xFF212121),
    this.onWeak = const Color(0xFFFFFFFF),
    this.onError = const Color(0xFFFFFFFF),
    this.onInfo = const Color(0xFFFFFFFF),
    this.onSuccess = const Color(0xFF212121),
    this.onWarning = const Color(0xFFFFFFFF),
    this.brightness = Brightness.light,
    this.appBarColor,
    this.scaffoldBackgroundColor,
    this.fontFamily,
  });

  /// Color definition for the theme.
  const ThemeColor.light({
    this.primary = Colors.blue,
    this.secondary = Colors.cyan,
    this.primaryContainer = Colors.blueAccent,
    this.secondaryContainer = Colors.cyanAccent,
    this.disabled = Colors.grey,
    this.weak = Colors.grey,
    this.error = Colors.red,
    this.warning = Colors.amber,
    this.info = Colors.blue,
    this.success = Colors.green,
    this.surface = const Color(0xFFFFFFFF),
    this.background = const Color(0xFFFFFFFF),
    this.onPrimary = const Color(0xFFFFFFFF),
    this.onSecondary = const Color(0xFFFFFFFF),
    this.onDisabled = const Color(0xFFFFFFFF),
    this.onSurface = const Color(0xFF212121),
    this.onBackground = const Color(0xFF212121),
    this.onWeak = const Color(0xFFFFFFFF),
    this.onError = const Color(0xFFFFFFFF),
    this.onInfo = const Color(0xFFFFFFFF),
    this.onSuccess = const Color(0xFFFFFFFF),
    this.onWarning = const Color(0xFFFFFFFF),
    this.appBarColor,
    this.scaffoldBackgroundColor,
    this.fontFamily,
  }) : brightness = Brightness.light;

  /// Color definition for the theme.
  const ThemeColor.dark({
    this.primary = Colors.blue,
    this.secondary = Colors.cyan,
    this.primaryContainer = Colors.blueAccent,
    this.secondaryContainer = Colors.cyanAccent,
    this.disabled = Colors.grey,
    this.weak = Colors.grey,
    this.error = Colors.red,
    this.warning = Colors.amber,
    this.info = Colors.blue,
    this.success = Colors.green,
    this.surface = const Color(0xFF212121),
    this.background = const Color(0xFF212121),
    this.onPrimary = const Color(0xFFFFFFFF),
    this.onSecondary = const Color(0xFFFFFFFF),
    this.onDisabled = const Color(0xFFFFFFFF),
    this.onSurface = const Color(0xFFFFFFFF),
    this.onBackground = const Color(0xFFFFFFFF),
    this.onWeak = const Color(0xFFFFFFFF),
    this.onError = const Color(0xFFFFFFFF),
    this.onInfo = const Color(0xFFFFFFFF),
    this.onSuccess = const Color(0xFFFFFFFF),
    this.onWarning = const Color(0xFFFFFFFF),
    this.appBarColor,
    this.scaffoldBackgroundColor,
    this.fontFamily,
  }) : brightness = Brightness.dark;

  /// Default font family.
  final String? fontFamily;

  /// Primary color.
  final Color primary;

  /// Secondary color.
  final Color secondary;

  /// Primary color container.
  final Color primaryContainer;

  /// Secondary color container.
  final Color secondaryContainer;

  /// The color when disabled.
  final Color disabled;

  /// The color of the outside line and the dividing line.
  final Color weak;

  /// Error color.
  final Color error;

  /// Attention color.
  final Color warning;

  /// Information color.
  final Color info;

  /// Color when successful.
  final Color success;

  /// Background color of dialogs, etc.
  final Color surface;

  /// Background color.
  final Color background;

  /// Primary color text color.
  final Color onPrimary;

  /// Secondary color text color.
  final Color onSecondary;

  /// Invalid text color.
  final Color onDisabled;

  /// Character color of the surface.
  final Color onSurface;

  /// Text color for the background color.
  final Color onBackground;

  /// Text color for lines.
  final Color onWeak;

  /// Text color for errors.
  final Color onError;

  /// Text color for information.
  final Color onInfo;

  /// Text color for success color.
  final Color onSuccess;

  /// Text color for attention color.
  final Color onWarning;

  /// Brightness.
  final Brightness brightness;

  /// App bar color.
  final Color? appBarColor;

  /// Scaffold color.
  final Color? scaffoldBackgroundColor;

  /// Conversion to theme data.
  ThemeData toThemeData() {
    switch (brightness) {
      case Brightness.dark:
        final colorScheme = const ColorScheme.dark().copyWith(
          primary: primary,
          primaryContainer: primaryContainer,
          secondary: secondary,
          secondaryContainer: secondaryContainer,
          surface: surface,
          background: background,
          error: error,
          onPrimary: onPrimary,
          onSecondary: onSecondary,
          onBackground: onBackground,
          onSurface: onSurface,
          onError: onError,
          brightness: brightness,
        );
        final theme = ThemeData.dark();
        return theme.copyWith(
          platform: TargetPlatform.iOS,
          colorScheme: colorScheme,
          splashColor: const Color(0xFFFFFFFF).withOpacity(0.8),
          scaffoldBackgroundColor: scaffoldBackgroundColor,
          textTheme: theme.textTheme.apply(
            bodyColor: onBackground,
            displayColor: onBackground,
            fontFamily: fontFamily,
          ),
          appBarTheme: appBarColor != null
              ? theme.appBarTheme.copyWith(
                  backgroundColor: appBarColor,
                  elevation: appBarColor == Colors.transparent ? 0 : null,
                  foregroundColor:
                      appBarColor == Colors.transparent ? onBackground : null,
                  toolbarTextStyle: appBarColor == Colors.transparent
                      ? theme.appBarTheme.toolbarTextStyle?.copyWith(
                          color: onBackground,
                        )
                      : null,
                  titleTextStyle: appBarColor == Colors.transparent
                      ? theme.appBarTheme.titleTextStyle?.copyWith(
                          color: onBackground,
                        )
                      : null,
                  iconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                  actionsIconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                )
              : null,
          buttonTheme: theme.buttonTheme.copyWith(
            textTheme: theme.buttonTheme.textTheme,
            buttonColor: primary,
            disabledColor: disabled,
            colorScheme: colorScheme,
          ),
          inputDecorationTheme: theme.inputDecorationTheme.copyWith(
            labelStyle: TextStyle(color: weak),
            helperStyle: TextStyle(color: weak),
            hintStyle: TextStyle(color: weak),
            counterStyle: TextStyle(color: weak),
            errorStyle: TextStyle(color: error),
            prefixStyle: TextStyle(color: onBackground),
            suffixStyle: TextStyle(color: onBackground),
            border: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            disabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(color: primary, width: 2),
            ),
            errorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
          ),
          indicatorColor: secondary,
          brightness: brightness,
          primaryColor: primary,
          disabledColor: disabled,
          dividerColor: weak,
          errorColor: error,
          backgroundColor: background,
          canvasColor: background,
        );
      default:
        final colorScheme = const ColorScheme.light().copyWith(
            primary: primary,
            primaryContainer: primaryContainer,
            secondary: secondary,
            secondaryContainer: secondaryContainer,
            surface: surface,
            background: background,
            error: error,
            onPrimary: onPrimary,
            onSecondary: onSecondary,
            onBackground: onBackground,
            onSurface: onSurface,
            onError: onError,
            brightness: brightness);
        final theme = ThemeData.light();
        return theme.copyWith(
          platform: TargetPlatform.iOS,
          colorScheme: colorScheme,
          splashColor: const Color(0xFFFFFFFF).withOpacity(0.8),
          scaffoldBackgroundColor: scaffoldBackgroundColor,
          appBarTheme: appBarColor != null
              ? theme.appBarTheme.copyWith(
                  backgroundColor: appBarColor,
                  elevation: appBarColor == Colors.transparent ? 0 : null,
                  foregroundColor:
                      appBarColor == Colors.transparent ? onBackground : null,
                  toolbarTextStyle: appBarColor == Colors.transparent
                      ? theme.appBarTheme.toolbarTextStyle?.copyWith(
                          color: onBackground,
                        )
                      : null,
                  titleTextStyle: appBarColor == Colors.transparent
                      ? theme.appBarTheme.titleTextStyle?.copyWith(
                          color: onBackground,
                        )
                      : null,
                  iconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                  actionsIconTheme: appBarColor == Colors.transparent
                      ? theme.iconTheme.copyWith(
                          color: onBackground,
                        )
                      : null,
                )
              : null,
          textTheme: theme.textTheme.apply(
            bodyColor: onBackground,
            displayColor: onBackground,
            fontFamily: fontFamily,
          ),
          buttonTheme: theme.buttonTheme.copyWith(
            buttonColor: primary,
            disabledColor: disabled,
            colorScheme: colorScheme,
          ),
          inputDecorationTheme: theme.inputDecorationTheme.copyWith(
            labelStyle: TextStyle(color: weak),
            helperStyle: TextStyle(color: weak),
            hintStyle: TextStyle(color: weak),
            counterStyle: TextStyle(color: weak),
            errorStyle: TextStyle(color: error),
            prefixStyle: TextStyle(color: onBackground),
            suffixStyle: TextStyle(color: onBackground),
            border: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            enabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            disabledBorder: OutlineInputBorder(
              borderSide: BorderSide(color: weak, width: 2),
            ),
            focusedBorder: OutlineInputBorder(
              borderSide: BorderSide(color: primary, width: 2),
            ),
            errorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
            focusedErrorBorder: OutlineInputBorder(
              borderSide: BorderSide(color: error, width: 2),
            ),
          ),
          indicatorColor: secondary,
          brightness: brightness,
          primaryColor: primary,
          disabledColor: disabled,
          dividerColor: weak,
          errorColor: error,
          backgroundColor: background,
          canvasColor: background,
        );
    }
  }
}
